==> api-service-common-0.12.28.tgz: package/src/middleware/apidocs.ts <==
/*************************************************************************
 *
 * Troven CONFIDENTIAL
 * __________________
 *
 *  (c) 2017-2020 Troven Pty Ltd
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Troven Pty Ltd and its licensors,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Troven Pty Ltd
 * and its suppliers and may be covered by International and Regional Patents,
 * patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Troven Pty Ltd.
 */

import * as _ from "lodash";
import { IOperation, IChassisContext, IChassisFeature, Operation, OpenAPI, OpenAPIPlugin} from "api-service-core";

/**
 * API Docs
 * --------
 * Attach an API resource to the OpenAPI definition.
 * We redact all of the "x-" attributes - unless disabled.
 *
 * @type {{name: string, description: string, title: string, usecase: [string], defaults: {path: string}, fn: module.exports.fn}}
 */
export default class apidocs implements IChassisFeature {

    name = "api.apidocs";
    title = "serve the OpenAPI spec";

    defaults = {
        path: "/swagger"
    };
    static BLACKLIST = [ "feature", "chassis" ];

    static redact = function(obj, allow_x?: boolean) {
        let must_redact = (k:string) => {
            return (!allow_x && k.startsWith("x-")) || this.BLACKLIST.indexOf(k)>=0;
        }
        _.each(obj, function(v, k) {
            if (_.isString(k) && must_redact(k) )  {
                delete obj[k];
            } else if ( _.isFunction(v)) {
                delete obj[k];
            } else if (_.isString(k) && must_redact(k) )  {
                delete obj[k];
            } else if ( Array.isArray(v)) {
                _.each(v, function(i) {
                    apidocs.redact(i, allow_x);
                });
            } else if ( _.isObject(v)) {
                apidocs.redact(v, allow_x);
            }
        });
        return obj;
    };

    static document = function(context: IChassisContext, openapi: OpenAPI, options: any) {
        // clean-up Open API info

        let _info = _.extend({ title: context.pkg.name + " (v "+context.pkg.version+")", version: "0.0.0" }, openapi.spec.info );
        let spec = { openapi: "3.0.0", info: _info, paths: {}, tags: {}, components: { schemas: {}, securitySchemes: {} } };

        // merge

        spec.tags = openapi.getTags();

        // console.log("spec.tags: %j", spec.tags);

        let paths = openapi.paths.operations();
        _.each( paths, (op: Operation) => {
            let path_spec = _.extend({ operationId: op.operationId, summary: op.actionId+" "+op.resource, description: "" }, op.spec );
            spec.paths[op.resource] = spec.paths[op.resource] || {};
            path_spec = spec.paths[op.resource][op.actionId] = spec.paths[op.resource][op.actionId] || path_spec;
            // path_spec.tags = ["test"]
        });

        let schemas = openapi.schemas.all();
        _.each(schemas, (_schema, name) => {
            let id = _schema.$id || name;
            let schema = _.extend({}, _schema);
            delete schema.$id;
            spec.components.schemas[id] = schema;
        });

        spec.components.securitySchemes = _.extend({},openapi.spec.components.securitySchemes);
        return this.redact(spec, options.allow_x);
    };

    fn(operation: IOperation, _options: any) {
        let context = operation.context;
        let plugin = context.plugins.get("openapi") as OpenAPIPlugin;
        let options = _.extend({ allow_x: true}, _options)

        let allow_x = options.allow_x || false;
        return function (_req: any, res: any) {

            console.log("REQUEST: %j %j", allow_x,_req.query);
            if (allow_x && _req.query.allow_x) allow_x = true;
            else allow_x = false;

            let openapi: OpenAPI = plugin.openapi;
            res.status(200);
            res.send( apidocs.document( context, openapi, { allow_x: allow_x } ) );
        };
    }
}

==> api-service-common-0.16.13.tgz: package/src/middleware/apidocs.ts <==
/*************************************************************************
 *
 * Troven CONFIDENTIAL
 * __________________
 *
 *  (c) 2017-2020 Troven Pty Ltd
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Troven Pty Ltd and its licensors,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Troven Pty Ltd
 * and its suppliers and may be covered by International and Regional Patents,
 * patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Troven Pty Ltd.
 */

import * as _ from "lodash";
import { IOperation, IChassisContext, IChassisFeature, Operation, OpenAPI, OpenAPIPlugin} from "api-service-core";

/**
 * API Docs
 * --------
 * Attach an API resource to the OpenAPI definition.
 * We redact all of the "x-" attributes - unless disabled.
 *
 * @type {{name: string, description: string, title: string, usecase: [string], defaults: {path: string}, fn: module.exports.fn}}
 */
export default class apidocs implements IChassisFeature {

    name = "api.apidocs";
    title = "serve the OpenAPI spec";

    defaults = {
        path: "/swagger"
    };
    static BLACKLIST = [ "feature", "chassis" ];

    static redact = function(obj, allow_x?: boolean) {
        let must_redact = (k:string) => {
            return (!allow_x && k.startsWith("x-")) || this.BLACKLIST.indexOf(k)>=0;
        }
        _.each(obj, function(v, k) {
            if (_.isString(k) && must_redact(k) )  {
                delete obj[k];
            } else if ( _.isFunction(v)) {
                delete obj[k];
            } else if (_.isString(k) && must_redact(k) )  {
                delete obj[k];
            } else if ( Array.isArray(v)) {
                _.each(v, function(i) {
                    apidocs.redact(i, allow_x);
                });
            } else if ( _.isObject(v)) {
                apidocs.redact(v, allow_x);
            }
        });
        return obj;
    };

    static document = function(context: IChassisContext, openapi: OpenAPI, options: any) {
        // clean-up Open API info

        let _info = _.extend({ title: context.pkg.name + " (v "+context.pkg.version+")", version: "0.0.0" }, openapi.spec.info );
        let spec = { openapi: "3.0.0", info: _info, paths: {}, tags: {}, components: { schemas: {}, securitySchemes: {} } };

        // merge

        spec.tags = openapi.getTags();

        // console.log("spec.tags: %j", spec.tags);

        let paths = openapi.paths.operations();
        _.each( paths, (op: Operation) => {
            let path = op.spec.path;
            let path_spec = _.extend({ operationId: op.operationId, summary: op.actionId+" "+op.resource, description: "" }, op.spec );
            spec.paths[path] = spec.paths[path] || {};
            delete path_spec.path;
            path_spec = spec.paths[path][op.actionId] = spec.paths[path][op.actionId] || path_spec;
            // path_spec.tags = ["test"]
        });

        let schemas = openapi.schemas.all();
        _.each(schemas, (_schema, name) => {
            let id = _schema.$id || name;
            let schema = _.extend({}, _schema);
            delete schema.$id;
            spec.components.schemas[id] = schema;
        });

        spec.components.securitySchemes = _.extend({},openapi.spec.components.securitySchemes);
        return this.redact(spec, options.allow_x);
    };

    fn(operation: IOperation, _options: any) {
        let context = operation.context;
        let plugin = context.plugins.get("openapi") as OpenAPIPlugin;
        let options = _.extend({ allow_x: true}, _options)

        let allow_x = options.allow_x || false;
        return function (_req: any, res: any) {

            if (allow_x && _req.query.allow_x) allow_x = true;
            else allow_x = false;

            let openapi: OpenAPI = plugin.openapi;
            res.status(200);
            res.send( apidocs.document( context, openapi, { allow_x: allow_x } ) );
        };
    }
}
